(* Comments van Yannick Lefevre

Dit is een aangepaste versie van Branco's StandardBitmap (zie Comments hieronder)
De veranderingen zijn:
	-DrawSavedBitmap is zo aangepast dat het groen (2) niet tekent,
	waardoor er dus met een achtergrondkleur kan gewerkt worden
	
	-De nieuwe procedure DrawBorderBitmap tekent enkel alleen wat zwart (0) was in de bitmap.
	En dit in de kleur die de gebruiker meegeeft als parameter van de proc *)
(*************************************************************************************)

(* !
$$$ Comments van Branko Brackx $$$

april 2006

Deze versie van StandardBitmap heeft volgende aanpassingen
ten opzichte van de versie van maart 2006 op parallel.vub.ac.be :

1) 1 pixelcorrectie in x en y richting
   De bitmaps werden verkeerd geplaatst en herplaatst,
   dit verklaart waarom een heel aantal spelletjes fouten vertonen zoals achterblijvende strepen.
   Betreft: procedures DrawBitmapFile en DrawSavedBitmap

2) De procedure DrawBitmapFile is sterk geoptimaliseerd door niet pixel per pixel,
   maar daarentegen lijnen te plotten.
   Resultaat: ongeveer 500 % snelheidswinst, afhankelijk van de inhoud van de bitmap.

! *)




<* WOFF + *>
IMPLEMENTATION MODULE ModifiedStandardBitmap;
FROM IO IMPORT WrCard, WrStr, WrInt, WrLn;
FROM RealMath IMPORT power;
FROM Graph IMPORT Plot, Line;
FROM Graph IMPORT _clrBLACK, _clrBLUE, _clrGREEN, _clrCYAN, _clrRED, _clrMAGENTA, _clrBROWN, _clrWHITE, _clrGRAY,   _clrLIGHTBLUE, _clrLIGHTGREEN, _clrLIGHTCYAN, _clrLIGHTRED, _clrLIGHTMAGENTA,  _clrLIGHTYELLOW, _clrBRIGHTWHITE;
FROM Graph IMPORT GetImage, PutImage, _GPSET; (* for bitmap saving *)
FROM Lib IMPORT Delay;
IMPORT FIO;

TYPE
Bitmap = ARRAY [1..BITMAP_SIZE],[1..BITMAP_SIZE] OF CHAR;
IconMap = ARRAY [1..DIGIT_SIZE],[1..DIGIT_SIZE] OF CHAR;

(* for optimisation: bitmap saving in graphics memory*)
BitmapSaveRc = RECORD
saved: BOOLEAN; (* is bitmap saved? *)
id: CARDINAL;   (* number of saved bitmap *)
      END;

VAR
gColorMap: ARRAY['a'..'z'] OF ColourSb; (* letter to colour correspondence mapping *)
gBitmaps: ARRAY BitmapEn OF Bitmap;
gSavedBitmaps : ARRAY BitmapEn, ColourSb OF BitmapSaveRc;
gDigitIcons: ARRAY DigitSb OF IconMap;
gSavedDigitIcons: ARRAY DigitSb, ColourSb, ColourSb OF BitmapSaveRc;

PROCEDURE DrawBitmap(x, y : CARDINAL; bitmap : BitmapEn; backgroundColor: ColourSb);
VAR
i, j : CARDINAL;
BEGIN

IF NOT gSavedBitmaps[bitmap][backgroundColor].saved OR (backgroundColor = 16) THEN (* first time to draw this bitmap & background *)

  gColorMap['z'] := backgroundColor;
  FOR i := 1 TO BITMAP_SIZE DO
  FOR j := 1 TO BITMAP_SIZE DO
    IF (gBitmaps[bitmap][i][j] >= 'a') AND (gBitmaps[bitmap][i][j] <= 'z') AND (gColorMap[gBitmaps[bitmap][i][j]] # 16) THEN
    Plot (x+j, y+i, gColorMap[gBitmaps[bitmap][i][j]]);
    END;
  END;
  END;
  GetImage(x+1, y+1, x + BITMAP_SIZE, y + BITMAP_SIZE, gSavedBitmaps[bitmap][backgroundColor].id); (* save bitmap into memory *)
  gSavedBitmaps[bitmap][backgroundColor].saved := TRUE;
ELSE
  PutImage(x+1, y+1, gSavedBitmaps[bitmap][backgroundColor].id, _GPSET);
END;

END DrawBitmap;

PROCEDURE DrawDigit(x, y : CARDINAL; digit : DigitSb; digitColor, backgroundColor: ColourSb);
  VAR i, j: CARDINAL;
BEGIN

IF NOT gSavedDigitIcons[digit][digitColor][backgroundColor].saved OR (digitColor = 16) OR (backgroundColor = 16) THEN (* first time to draw this bitmap & background *)

  FOR i := 1 TO DIGIT_SIZE DO
    FOR j := 1 TO DIGIT_SIZE DO
      IF gDigitIcons[digit][i][j] = 'a' THEN
        IF backgroundColor # 16 THEN
          Plot(x+j, y+i, backgroundColor);
      END;
      ELSE
        IF digitColor # 16 THEN
          Plot(x+j, y+i, digitColor);
       END;
      END;
    END;
  END;

  GetImage(x+1, y+1, x + DIGIT_SIZE, y + DIGIT_SIZE, gSavedDigitIcons[digit][digitColor][backgroundColor].id); (* save bitmap into memory *)
  gSavedDigitIcons[digit][digitColor][backgroundColor].saved := TRUE;
ELSE
  PutImage(x+1, y+1, gSavedDigitIcons[digit][digitColor][backgroundColor].id, _GPSET);
END;

END DrawDigit;

PROCEDURE DrawNumber(x, y : CARDINAL; number : CARDINAL; numberColor, backgroundColor: ColourSb);
  VAR
    nbrDigits, i, digit, rest: CARDINAL;
    xtopleft, x0, y0: CARDINAL;
BEGIN
  rest := number;
  nbrDigits := 0;
  REPEAT
    INC(nbrDigits);
    number := number / 10;
  UNTIL (number = 0);
  xtopleft := x - (nbrDigits * DIGIT_SIZE )/2;
  FOR i := 1 TO nbrDigits DO
    digit := rest / VAL(CARDINAL, power(10.0, VAL(REAL, nbrDigits - i)));
    rest := rest MOD VAL(CARDINAL, power(10.0, VAL(REAL, nbrDigits - i)));
    x0 := xtopleft + (i - 1) * DIGIT_SIZE;
    y0 := y - DIGIT_SIZE/2;
    DrawDigit(x0, y0, digit, numberColor, backgroundColor);
  END;
END DrawNumber;

(* bmp files *)
PROCEDURE Get_Real_Color(hexcolor : CARDINAL) : CARDINAL;

VAR color : CARDINAL;

BEGIN
CASE hexcolor OF

  0  : color := 0;   (* black *)
| 1  : color := 1;   (* brown *)
| 2  : color := 2;   (* dark green *)
| 3  : color := 3;   (* kaki *)
| 4  : color := 4;   (* dark blue *)
| 5  : color := 5;   (* purple *)
| 6  : color := 6;   (* turquois *)
| 15 : color := 10;  (* white *)
| 8  : color := 11;  (* light grey *)
| 7  : color := 12;  (* dark grey *)
| 9  : color := 13;  (* red *)
| 10 : color := 14;  (* light green *)
| 11 : color := 15;  (* yellow *)
| 12 : color := 16;  (* blue *)
| 13 : color := 17;  (* pink *)
| 14 : color := 18;  (* light blue *)

ELSE color := 0;
END;

RETURN color;

END Get_Real_Color;


PROCEDURE DrawBitmapFile(position_x, position_y : CARDINAL; bitmapname : ARRAY OF CHAR; VAR width, height : CARDINAL):CARDINAL;

CONST POS_PIX_X = 18;
  POS_PIX_Y = 22;
  POS_COLOR = 118;

VAR bitmapfile : FIO.File;
  hexcolor1, hexcolor2, ascii_char, color : CARDINAL;
  counter_x, counter_y, vorigekleur, kleurteller, paaltje, spaces, position : CARDINAL;
(*width, height : CARDINAL;*)
char_1, char_2, temp : CHAR;
         id:CARDINAL;
BEGIN

  IF NOT FIO.Exists(bitmapname) THEN
    WrStr("File ");WrStr(bitmapname);WrStr(" does NOT exist!");WrLn;
    id := 0;
  ELSE
    bitmapfile := FIO.OpenRead(bitmapname);

    FIO.Seek(bitmapfile, POS_PIX_X);
    char_1 :=FIO.RdChar(bitmapfile);
    char_2 :=FIO.RdChar(bitmapfile);
    width := 256 * ORD(char_2) + ORD(char_1);

    FIO.Seek(bitmapfile, POS_PIX_Y);
    char_1 :=FIO.RdChar(bitmapfile);
    char_2 :=FIO.RdChar(bitmapfile);
    height := 256 * ORD(char_2) + ORD(char_1);

    FIO.Seek(bitmapfile, POS_COLOR);

    counter_y := 0;
    WHILE FIO.EOF = FALSE DO
      temp := FIO.RdChar(bitmapfile);
      counter_y := counter_y +1;
    END;

    IF (counter_y < (width * height DIV 2)) THEN
      WrStr("Bitmap file ");WrStr(bitmapname);WrStr(" is not a 16-colour bitmap... so cannot be drawn");
      id := 0;
    ELSE
    spaces := ((counter_y -(width * height DIV 2)) DIV height);
              (* (aantalbytes - aantalpixels/2) / hoogte = aantal lege bytes na een lijn *)
    FIO.Seek(bitmapfile, POS_COLOR);

      FOR counter_y := 0 TO height - 1 DO

        counter_x := 0;
        WHILE (counter_x < width) DO

          ascii_char := ORD(FIO.RdChar(bitmapfile));
          hexcolor1 := ascii_char DIV 16;
          color := Get_Real_Color(hexcolor1);


    (* eerste punt van de lijn *)
    IF counter_x = 0 THEN
      IF color#2 THEN
        Plot(position_x, position_y + height - counter_y - 1, color);
      END;
      counter_x := 1;
      vorigekleur := color;
      kleurteller := 1;
      paaltje := 0;
    ELSE
    (* block, idem hieronder *)	
      IF (color <> vorigekleur) OR (counter_x = width -1) THEN
      (* teken punt of lijn *)
        IF vorigekleur#2 THEN
      	   IF (kleurteller = 1) THEN
               Plot(position_x + counter_x - 1, position_y + height - counter_y - 1, vorigekleur);      (* -1 toegev. *)
           ELSE
               Line(position_x + paaltje, position_y + height - counter_y - 1, position_x + counter_x - 1, position_y + height - counter_y - 1,vorigekleur);
           END;
	END;
        paaltje := counter_x;
        vorigekleur := color;
        kleurteller := 1;
      ELSE
        INC(kleurteller);
      END;
      IF ((counter_x = width - 1) OR (counter_x = width - 2)) AND (color#2) THEN
        Plot(position_x + counter_x, position_y + height - counter_y - 1, color);
      END;
      INC(counter_x);
      (* end block *)
    END;

    IF (counter_x < width) THEN
      hexcolor2 := ascii_char MOD 16;
      color := Get_Real_Color(hexcolor2);

    (* block *)	
      IF (color <> vorigekleur) OR (counter_x = width - 1) THEN
      (* teken punt of lijn *)
        IF vorigekleur#2 THEN
          IF (kleurteller = 1) THEN
            Plot(position_x + counter_x - 1, position_y + height - counter_y - 1, vorigekleur);      (* -1 toegev. *)
          ELSE
            Line(position_x + paaltje, position_y + height - counter_y - 1, position_x + counter_x - 1, position_y + height - counter_y - 1,vorigekleur);
          END;
	END;
        paaltje := counter_x;
        vorigekleur := color;
        kleurteller := 1;
      ELSE
        INC(kleurteller);
      END;
      IF ((counter_x = width - 1) OR (counter_x = width - 2)) AND (color#2) THEN
        Plot(position_x + counter_x, position_y + height - counter_y - 1, color);
      END;
      INC(counter_x);
      (* end block *)
    END;

  END;

  position := FIO.GetPos(bitmapfile);
  FIO.Seek(bitmapfile, position +spaces);

END;

GetImage(position_x, position_y, position_x + width - 1, position_y + height - 1, id); (* 2x -1 toegev. *)
    END;
    FIO.Close(bitmapfile);
  END;
  RETURN id;
END DrawBitmapFile;

PROCEDURE DrawBorderBitmapFile(position_x, position_y : CARDINAL; bitmapname : ARRAY OF CHAR; BorderColor : CARDINAL );

CONST POS_PIX_X = 18;
  POS_PIX_Y = 22;
  POS_COLOR = 118;

VAR bitmapfile : FIO.File;
  hexcolor1, hexcolor2, ascii_char, color : CARDINAL;
  counter_x, counter_y, vorigekleur, kleurteller, paaltje, spaces, position : CARDINAL;
  width, height : CARDINAL;
char_1, char_2, temp : CHAR;
BEGIN

  IF NOT FIO.Exists(bitmapname) THEN
    WrStr("File ");WrStr(bitmapname);WrStr(" does NOT exist!");WrLn;
  ELSE
    bitmapfile := FIO.OpenRead(bitmapname);

    FIO.Seek(bitmapfile, POS_PIX_X);
    char_1 :=FIO.RdChar(bitmapfile);
    char_2 :=FIO.RdChar(bitmapfile);
    width := 256 * ORD(char_2) + ORD(char_1);

    FIO.Seek(bitmapfile, POS_PIX_Y);
    char_1 :=FIO.RdChar(bitmapfile);
    char_2 :=FIO.RdChar(bitmapfile);
    height := 256 * ORD(char_2) + ORD(char_1);

    FIO.Seek(bitmapfile, POS_COLOR);

    counter_y := 0;
    WHILE FIO.EOF = FALSE DO
      temp := FIO.RdChar(bitmapfile);
      counter_y := counter_y +1;
    END;

    IF (counter_y < (width * height DIV 2)) THEN
      WrStr("Bitmap file ");WrStr(bitmapname);WrStr(" is not a 16-colour bitmap... so cannot be drawn");
    ELSE
    spaces := ((counter_y -(width * height DIV 2)) DIV height);
              (* (aantalbytes - aantalpixels/2) / hoogte = aantal lege bytes na een lijn *)
    FIO.Seek(bitmapfile, POS_COLOR);

      FOR counter_y := 0 TO height - 1 DO

        counter_x := 0;
        WHILE (counter_x < width) DO

          ascii_char := ORD(FIO.RdChar(bitmapfile));
          hexcolor1 := ascii_char DIV 16;
          color := Get_Real_Color(hexcolor1);


    (* eerste punt van de lijn *)
    IF counter_x = 0 THEN
      IF color=0 THEN
        Plot(position_x, position_y + height - counter_y - 1, BorderColor);
      END;
      counter_x := 1;
      vorigekleur := color;
      kleurteller := 1;
      paaltje := 0;
    ELSE
    (* block, idem hieronder *)	
      IF (color <> vorigekleur) OR (counter_x = width -1) THEN
      (* teken punt of lijn *)
        IF vorigekleur=0 THEN
      	   IF (kleurteller = 1) THEN
               Plot(position_x + counter_x - 1, position_y + height - counter_y - 1, BorderColor);      (* -1 toegev. *)
           ELSE
               Line(position_x + paaltje, position_y + height - counter_y - 1, position_x + counter_x - 1, position_y + height - counter_y - 1,BorderColor);
           END;
	END;
        paaltje := counter_x;
        vorigekleur := color;
        kleurteller := 1;
      ELSE
        INC(kleurteller);
      END;
      IF ((counter_x = width - 1) OR (counter_x = width - 2)) AND (color=0) THEN
        Plot(position_x + counter_x, position_y + height - counter_y - 1, BorderColor);
      END;
      INC(counter_x);
      (* end block *)
    END;

    IF (counter_x < width) THEN
      hexcolor2 := ascii_char MOD 16;
      color := Get_Real_Color(hexcolor2);

    (* block *)	
      IF (color <> vorigekleur) OR (counter_x = width - 1) THEN
      (* teken punt of lijn *)
        IF vorigekleur=0 THEN
          IF (kleurteller = 1) THEN
            Plot(position_x + counter_x - 1, position_y + height - counter_y - 1, BorderColor);      (* -1 toegev. *)
          ELSE
            Line(position_x + paaltje, position_y + height - counter_y - 1, position_x + counter_x - 1, position_y + height - counter_y - 1,BorderColor);
          END;
	END;
        paaltje := counter_x;
        vorigekleur := color;
        kleurteller := 1;
      ELSE
        INC(kleurteller);
      END;
      IF ((counter_x = width - 1) OR (counter_x = width - 2)) AND (color=0) THEN
        Plot(position_x + counter_x, position_y + height - counter_y - 1, BorderColor);
      END;
      INC(counter_x);
      (* end block *)
    END;

  END;

  position := FIO.GetPos(bitmapfile);
  FIO.Seek(bitmapfile, position +spaces);

END;

    END;
    FIO.Close(bitmapfile);
  END;
END DrawBorderBitmapFile;




PROCEDURE DrawSavedBitmap(position_x, position_y, bitmap_ID: CARDINAL);
BEGIN
PutImage(position_x, position_y, bitmap_ID, _GPSET);
END DrawSavedBitmap;
BEGIN
(* Init the color map *)

gColorMap['a'] := _clrBLACK         ; (*    0*)
gColorMap['b'] := _clrBLUE          ; (*    1*)
gColorMap['c'] := _clrGREEN         ; (*    2*)
gColorMap['d'] := _clrCYAN          ; (*    3*)
gColorMap['e'] := _clrRED           ; (*    4*)
gColorMap['f'] := _clrMAGENTA       ; (*    5*)
gColorMap['g'] := _clrBROWN         ; (*    6*)
gColorMap['h'] := _clrWHITE         ; (*    7*)
gColorMap['i'] := _clrGRAY          ; (*    8*)
gColorMap['j'] := _clrLIGHTBLUE     ; (*    9*)
gColorMap['k'] := _clrLIGHTGREEN    ;(*    10*)
gColorMap['l'] := _clrLIGHTCYAN     ;(*    11*)
gColorMap['m'] := _clrLIGHTRED      ;(*    12*)
gColorMap['n'] := _clrLIGHTMAGENTA  ;(*    13*)
gColorMap['o'] := _clrLIGHTYELLOW   ;(*    14*)
gColorMap['p'] := _clrBRIGHTWHITE   ;(*    15*)
(* 'z' is used for the background color *)

(* STEP 3: create the bitmap *)
(* initialise Bitmaps *)

(* Hunter *)
gBitmaps[HUNTER][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][5]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][6]:="zzzzzzzzzzzzzooooooozzzzzzzzzzzz";
gBitmaps[HUNTER][7]:="zzzzzzzzzzzooooooooooozzzzzzzzzz";
gBitmaps[HUNTER][8]:="zzzzzzzzzzoaaoooooooooozzzzzzzzz";
gBitmaps[HUNTER][9]:="zzzzzzzzzoooaaaaoooooooozzzzzzzz";
gBitmaps[HUNTER][10]:="zzzzzzzzoooooooaaaooooooozzzzzzz";
gBitmaps[HUNTER][11]:="zzzzzzzooooooooooaaaoooooozzzzzz";
gBitmaps[HUNTER][12]:="zzzzzzoooooooooooooaaaooooozzzzz";
gBitmaps[HUNTER][13]:="zzzzzzooooooeaoooooaaaaoooozzzzz";
gBitmaps[HUNTER][14]:="zzzzzooooooeeaaooooaaaaaaooozzzz";
gBitmaps[HUNTER][15]:="zzzzzooooooaaaaooooaaaaaaaaazzzz";
gBitmaps[HUNTER][16]:="zzzzzoooooooaaoooooaaaaooooazzzz";
gBitmaps[HUNTER][17]:="zzzzzooooooooooooooooooooooozzzz";
gBitmaps[HUNTER][18]:="zzzzzooooooooooooooooooooooozzzz";
gBitmaps[HUNTER][19]:="zzzzzoooooooooooooooooeooooozzzz";
gBitmaps[HUNTER][20]:="zzzzzoooooooooooooooioeooooozzzz";
gBitmaps[HUNTER][21]:="zzzzzziiiiiiiipppppiiieggggggzzz";
gBitmaps[HUNTER][22]:="zzzzeeiippppppiiiiiiieeggggggzzz";
gBitmaps[HUNTER][23]:="zzzzzziiiiiiiiiiiiiiiieggggggzzz";
gBitmaps[HUNTER][24]:="zzzzzzzzooooooooooooooeoozzzzzzz";
gBitmaps[HUNTER][25]:="zzzzzzzzzooooooooooooooozzzzzzzz";
gBitmaps[HUNTER][26]:="zzzzzzzzzzooooooooooooozzzzzzzzz";
gBitmaps[HUNTER][27]:="zzzzzzzzzzzooooooooooozzzzzzzzzz";
gBitmaps[HUNTER][28]:="zzzzzzzzzzzzzooooooozzzzzzzzzzzz";
gBitmaps[HUNTER][29]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][30]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[HUNTER][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* Ghost *)
gBitmaps[GHOST][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][5]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][6]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][7]:="zzzzzzzzzzzpppppppzzzzzzzzzzzzzz";
gBitmaps[GHOST][8]:="zzzzzzzzzzppppppppppzzzzzzzzzzzz";
gBitmaps[GHOST][9]:="zzzzzzzzzpppppppppppzzzzzzzzzzzz";
gBitmaps[GHOST][10]:="zzzzzzzzpppppppppppppzzzzzzzzzzz";
gBitmaps[GHOST][11]:="zzzzzzzzpppzzpppzzpppzzzzzzzzzzz";
gBitmaps[GHOST][12]:="zzzzzzzzpppzzpppzzpppzzzzzzzzzzz";
gBitmaps[GHOST][13]:="zzzzzzzzpppppppppppppzzzzzzzzzzz";
gBitmaps[GHOST][14]:="zzzzzzzzpppppppppppppzzzzpzzzzzz";
gBitmaps[GHOST][15]:="zzzzzpzzpppppzzppppppzzzzpppzzzz";
gBitmaps[GHOST][16]:="zzzpppzzppppzzzzpppppzzzppppzzzz";
gBitmaps[GHOST][17]:="zzzppppzpppppzzppppppzpppzzzzzzz";
gBitmaps[GHOST][18]:="zzzzzpppppppppppppppppppzzzzzzzz";
gBitmaps[GHOST][19]:="zzzzzzpppppppppppppppppzzzzzzzzz";
gBitmaps[GHOST][20]:="zzzzzzzppppppppppppppppzzzzzzzzz";
gBitmaps[GHOST][21]:="zzzzzzzpppppppppppppppzzzzzzzzzz";
gBitmaps[GHOST][22]:="zzzzzzzzppppppppppppppzzzzzzzzzz";
gBitmaps[GHOST][23]:="zzzzzzzpppppppppppppppzzzzzzzzzz";
gBitmaps[GHOST][24]:="zzzzzzzppppppppppppppppzzzzzzzzz";
gBitmaps[GHOST][25]:="zzzzzzpppppppppppppppppzzzzzzzzz";
gBitmaps[GHOST][26]:="zzzzzpppppppppppppppppppzzzzzzzz";
gBitmaps[GHOST][27]:="zzzzzpppppppppppppppppppzzzzzzzz";
gBitmaps[GHOST][28]:="zzzzzpppppppppppppppppppzzzzzzzz";
gBitmaps[GHOST][29]:="zzzzzzpppppppzzzzzzzzpppzzzzzzzz";
gBitmaps[GHOST][30]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[GHOST][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* Mine *)
gBitmaps[MINE][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[MINE][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[MINE][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[MINE][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[MINE][5]:="zzzzzzzzzzzzzzzizzzzzzzzzzzzzzzz";
gBitmaps[MINE][6]:="zzzzzzzzzzzzzziiizzzzzzzzzzzzzzz";
gBitmaps[MINE][7]:="zzzzzzzzzzzziiiiiiizzzzzzzzzzzzz";
gBitmaps[MINE][8]:="zzzzzzzzzziiiiiiiiiiizzzzzzzzzzz";
gBitmaps[MINE][9]:="zzzzzziiiiiiiiizzzzziiiiizzzzzzz";
gBitmaps[MINE][10]:="zzzzzziiiiiiizziiiiizziiizzzzzzz";
gBitmaps[MINE][11]:="zzzzzziiiiizziiiiiiiiizzizzzzzzz";
gBitmaps[MINE][12]:="zzzzzziiiiziiiiiiiiiiiiizzzzzzzz";
gBitmaps[MINE][13]:="zzzzziiiiiziiiiiiiiiiiiizizzzzzz";
gBitmaps[MINE][14]:="zzzzziiiiziiiiiiiiiiiiiiizzzzzzz";
gBitmaps[MINE][15]:="zzzziiiiiziiiiiiiiiiiiiiizizzzzz";
gBitmaps[MINE][16]:="zzzziiiiziiiiiieeeeeiiiiiizzzzzz";
gBitmaps[MINE][17]:="zzziiiiiziiiiiiepppeiiiiiizizzzz";
gBitmaps[MINE][18]:="zziiiiiiziiiiiiepepeiiiiiiziizzz";
gBitmaps[MINE][19]:="zzziiiiiziiiiiiepppeiiiiiizizzzz";
gBitmaps[MINE][20]:="zzzziiiiziiiiiieeeeeiiiiiizzzzzz";
gBitmaps[MINE][21]:="zzzziiiiiziiiiiiiiiiiiiiizizzzzz";
gBitmaps[MINE][22]:="zzzzziiiiziiiiiiiiiiiiiiizzzzzzz";
gBitmaps[MINE][23]:="zzzzziiiiiziiiiiiiiiiiiizizzzzzz";
gBitmaps[MINE][24]:="zzzzzziiiiziiiiiiiiiiiiizzzzzzzz";
gBitmaps[MINE][25]:="zzzzzziiiiizziiiiiiiiizzizzzzzzz";
gBitmaps[MINE][26]:="zzzzzziiiiiiizziiiiizziiizzzzzzz";
gBitmaps[MINE][27]:="zzzzzziiiiiiiiizzzzziiiiizzzzzzz";
gBitmaps[MINE][28]:="zzzzzzzzzziiiiiiiiiiizzzzzzzzzzz";
gBitmaps[MINE][29]:="zzzzzzzzzzzziiiiiiizzzzzzzzzzzzz";
gBitmaps[MINE][30]:="zzzzzzzzzzzzzziiizzzzzzzzzzzzzzz";
gBitmaps[MINE][31]:="zzzzzzzzzzzzzzzizzzzzzzzzzzzzzzz";
gBitmaps[MINE][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* Apple *)
gBitmaps[APPLE][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[APPLE][2]:="zzzzzzzzzzzzzzzzzzzzaaaazzzzzzzz";
gBitmaps[APPLE][3]:="zzzzzzzzzzzzzzzzzzaaaazzzzzzzzzz";
gBitmaps[APPLE][4]:="zzzzzzzzzzzzzzzzzaazzzzzzzzzzzzz";
gBitmaps[APPLE][5]:="zzzzzzzzzzzzzzzzaazzzzzzzzzzzzzz";
gBitmaps[APPLE][6]:="zzzzzzzzzzzzzzzaaazzzzzzzzzzzzzz";
gBitmaps[APPLE][7]:="zzzzzzzzzeeeeezaazeeeeezzzzzzzzz";
gBitmaps[APPLE][8]:="zzzzzzzeeeeeeeeeeeeeeeeeezzzzzzz";
gBitmaps[APPLE][9]:="zzzzzzzeeeeeeeeeeeeeeeeeeezzzzzz";
gBitmaps[APPLE][10]:="zzzzzeeeeeeeeeeeeeeeeeeeeezzzzzz";
gBitmaps[APPLE][11]:="zzzzeeeeeeeeeeeeeeeppeppeeezzzzz";
gBitmaps[APPLE][12]:="zzzzeeeeeeeeeeeeeeeppeeppeezzzzz";
gBitmaps[APPLE][13]:="zzzeeeeeeeeeeeeeeeeeppeeppeezzzz";
gBitmaps[APPLE][14]:="zzzeeeeeeeeeeeeeeeeepppeppeezzzz";
gBitmaps[APPLE][15]:="zzeeeeeeeeeeeeeeeeeeeepeeeeeezzz";
gBitmaps[APPLE][16]:="zzeeeeeeeeeeeeeeeeeeeeeeeeeeezzz";
gBitmaps[APPLE][17]:="zzzeeeeeeeeeeeeeeeeeeeeeeeeeezzz";
gBitmaps[APPLE][18]:="zzzeeeeeeeeeeeeeeeeeeeeeeeeeezzz";
gBitmaps[APPLE][19]:="zzzzeeeeeeeeeeeeeeeeeeeeeeeeezzz";
gBitmaps[APPLE][20]:="zzzzeeeeeeeeeeeeeeeeeeeeeeeezzzz";
gBitmaps[APPLE][21]:="zzzzeeeeeeeeeeeeeeeeeeeeeeeezzzz";
gBitmaps[APPLE][22]:="zzzzzeeeeeeeeeeeeeeeeeeeeeezzzzz";
gBitmaps[APPLE][23]:="zzzzzzeeeeeeeeeeeeeeeeeeeezzzzzz";
gBitmaps[APPLE][24]:="zzzzzzzeeeeeeeeeeeeeeeeeezzzzzzz";
gBitmaps[APPLE][25]:="zzzzzzzzeeeeeeeeeeeeeeezzzzzzzzz";
gBitmaps[APPLE][26]:="zzzzzzzzzzeeeeeeeeeeeezzzzzzzzzz";
gBitmaps[APPLE][27]:="zzzzzzzzzzzeeeeeeeezzzzzzzzzzzzz";
gBitmaps[APPLE][28]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[APPLE][29]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[APPLE][30]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[APPLE][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[APPLE][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* Eater *)
gBitmaps[EATER][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][5]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][6]:="zzzzzzzzzzzzzzbbbbbbbzzzzzzzzzzz";
gBitmaps[EATER][7]:="zzzzzzzzzzzzbbbbbbbbbbbzzzzzzzzz";
gBitmaps[EATER][8]:="zzzzzzzzzzbbbbbbbbbbbbbbbzzzzzzz";
gBitmaps[EATER][9]:="zzzzzzzzzbbbbbbbbbbbbbbbbbzzzzzz";
gBitmaps[EATER][10]:="zzzzzzzzbbbbbbbbbbbbbbbbbbbzzzzz";
gBitmaps[EATER][11]:="zzzzzzzbbbbbbbbbbbaaabbbbbbbzzzz";
gBitmaps[EATER][12]:="zzzzzzbbbbbbbbbbbbaaabbbbbbbzzzz";
gBitmaps[EATER][13]:="zzzzzzbbbbbbbbbbbbaapbbbbbbbbzzz";
gBitmaps[EATER][14]:="zzzzzbbbbbbbbbbbbbbbbbbbbbbbbzzz";
gBitmaps[EATER][15]:="zzzzzbbbbbbbbbbbbbbbbbbbbbbbzzzz";
gBitmaps[EATER][16]:="zzzzzbbbbbbbbbbbbbbbbbbbbbzzzzzz";
gBitmaps[EATER][17]:="zzzzzbbbbbbbbbbbbbbbbbbbzzzzzzzz";
gBitmaps[EATER][18]:="zzzzzbbbbbbbbbbbbbbbbzzzzzzzzzzz";
gBitmaps[EATER][19]:="zzzzzbbbbbbbbbbbbbbbzzzzzzzzzzzz";
gBitmaps[EATER][20]:="zzzzzbbbbbbbbbbbbbbbbzzzzzzzzzzz";
gBitmaps[EATER][21]:="zzzzzbbbbbbbbbbbbbbbbbzzzzzzzzzz";
gBitmaps[EATER][22]:="zzzzzzbbbbbbbbbbbbbbbbbzzzzzzzzz";
gBitmaps[EATER][23]:="zzzzzzbbbbbbbbbbbbbbbbbbbzzzzzzz";
gBitmaps[EATER][24]:="zzzzzzzbbbbbbbbbbbbbbbbbbbzzzzzz";
gBitmaps[EATER][25]:="zzzzzzzbbbbbbbbbbbbbbbbbbbbbzzzz";
gBitmaps[EATER][26]:="zzzzzzzzbbbbbbbbbbbbbbbbbbbzzzzz";
gBitmaps[EATER][27]:="zzzzzzzzzbbbbbbbbbbbbbbbbbzzzzzz";
gBitmaps[EATER][28]:="zzzzzzzzzzbbbbbbbbbbbbbbbzzzzzzz";
gBitmaps[EATER][29]:="zzzzzzzzzzzzbbbbbbbbbbbzzzzzzzzz";
gBitmaps[EATER][30]:="zzzzzzzzzzzzzzbbbbbbbzzzzzzzzzzz";
gBitmaps[EATER][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EATER][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* explorer *)
gBitmaps[EXPLORER][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][5]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][6]:="zzzzzzzzzzzzzeeeeeeeezzzzzzzzzzz";
gBitmaps[EXPLORER][7]:="zzzzzzzzzzzeeppeeeeppeezzzzzzzzz";
gBitmaps[EXPLORER][8]:="zzzzzzzzzzeeeppeeeepeeeeezzzzzzz";
gBitmaps[EXPLORER][9]:="zzezzzzzzeeeeeeeeeeeeeeeeezzzzzz";
gBitmaps[EXPLORER][10]:="eeezzzeeppeeeeeeeeebbbbbbbbzzzzz";
gBitmaps[EXPLORER][11]:="zeezzeeeppeeeeeebbaaabbbbbbbzzzz";
gBitmaps[EXPLORER][12]:="zzeeepeeeeeebbbbbbaaabbbbbbbzzzz";
gBitmaps[EXPLORER][13]:="zzzeppeeeebbbbbbbbaapbbbbbbbbzzz";
gBitmaps[EXPLORER][14]:="zeeeeeeebbbbbbbbbbbbbbbbbbbbbzzz";
gBitmaps[EXPLORER][15]:="eeeezeebbbbbbbbbbbbbbbbbbbbbbbzz";
gBitmaps[EXPLORER][16]:="eezzzbbbbbbbbbbbbbbbbbbbbbbbbbzz";
gBitmaps[EXPLORER][17]:="zzzzzbbbbbbbbbbbbbbbbbbbbbbbbbzz";
gBitmaps[EXPLORER][18]:="zzzzzbbbbbbbbbbbbbbbbbbbbbbbbbzz";
gBitmaps[EXPLORER][19]:="zzzzzbbbbbbbbbbbbbbbbbbbbbbbbbzz";
gBitmaps[EXPLORER][20]:="zzzzzbbbbbbbbbbbbbbbbbpppbbbbbzz";
gBitmaps[EXPLORER][21]:="zzzzzbbbbbbbbbbbbbbbbbbbbpppppzz";
gBitmaps[EXPLORER][22]:="zzzzzzbbbbbbbbbbbbbbbbbbbbbbbzzz";
gBitmaps[EXPLORER][23]:="zzzzzzbbbbbbbbbbbbbbbbbbbbbbbzzz";
gBitmaps[EXPLORER][24]:="zzzzzzzbbbbbbbbbbbbbbbbbbbbbbzzz";
gBitmaps[EXPLORER][25]:="zzzzzzzbbbbbbbbbbbbbbbbbbbbbzzzz";
gBitmaps[EXPLORER][26]:="zzzzzzzzbbbbbbbbbbbbbbbbbbbzzzzz";
gBitmaps[EXPLORER][27]:="zzzzzzzzzbbbbbbbbbbbbbbbbbzzzzzz";
gBitmaps[EXPLORER][28]:="zzzzzzzzzzbbbbbbbbbbbbbbbzzzzzzz";
gBitmaps[EXPLORER][29]:="zzzzzzzzzzzzbbbbbbbbbbbzzzzzzzzz";
gBitmaps[EXPLORER][30]:="zzzzzzzzzzzzzzbbbbbbbzzzzzzzzzzz";
gBitmaps[EXPLORER][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLORER][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* bonus *)
gBitmaps[BONUS][1]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][2]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][3]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][4]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][5]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][6]:="zzzzzzzzzzzzzzooooooozzzzzzzzzzz";
gBitmaps[BONUS][7]:="zzzzzzzzzzzooooooooooooozzzzzzzz";
gBitmaps[BONUS][8]:="zzzzzzzzzzooooooooooooooozzzzzzz";
gBitmaps[BONUS][9]:="zzzzzzzzzooooozzzzzzzooooozzzzzz";
gBitmaps[BONUS][10]:="zzzzzzzooooozzzzzzzzzzzoooozzzzz";
gBitmaps[BONUS][11]:="zzzzzzzoooozzzzzzzzzzzzzoooozzzz";
gBitmaps[BONUS][12]:="zzzzzzoooozzzzzzzzzzzzzzoooozzzz";
gBitmaps[BONUS][13]:="zzzzzoooozzzzzzzzzzzzzzzzoooozzz";
gBitmaps[BONUS][14]:="zzzzzoooozzzzzzzzzzzaaaaaoooozzz";
gBitmaps[BONUS][15]:="zzzooooooooooooooooooaaaaaaaazzz";
gBitmaps[BONUS][16]:="zzooooooooooooooooooaaaaaaaaaazz";
gBitmaps[BONUS][17]:="zzzzooooozzzzzzzaaaaaazzzzaaaaaz";
gBitmaps[BONUS][18]:="zzzzooooozzzzzaaaaaaaaazzzzaaaaa";
gBitmaps[BONUS][19]:="zzzooooooooooooooooooazzzzzzzzzz";
gBitmaps[BONUS][20]:="zzoooooooooooooooooozzzzzzzzzzzz";
gBitmaps[BONUS][21]:="zzzzzooooozzaaaazzzzzzzzzzzzzaaa";
gBitmaps[BONUS][22]:="zzzzzzoooozaaaazzzzzzzzzzzzzaaaa";
gBitmaps[BONUS][23]:="zzzzzzoooooaaaaaaaaazzzzzzzaaaaz";
gBitmaps[BONUS][24]:="zzzzzzoooooaaaaaaaazzzzzooooaaaz";
gBitmaps[BONUS][25]:="zzzzzzzoooooaazzzzzzzzzoooooaazz";
gBitmaps[BONUS][26]:="zzzzzzzzoooooazzzzzzzzoooooaazzz";
gBitmaps[BONUS][27]:="zzzzzzzzzoooooazzzzzzooooooazzzz";
gBitmaps[BONUS][28]:="zzzzzzzzzzoooooooooooooooaazzzzz";
gBitmaps[BONUS][29]:="zzzzzzzzzzzooooooooooooazzzzzzzz";
gBitmaps[BONUS][30]:="zzzzzzzzzzzzzzooooooozzzzzzzzzzz";
gBitmaps[BONUS][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[BONUS][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* gBitmaps[EXPLOSION] *)
gBitmaps[EXPLOSION][1] :="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][2] :="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][3] :="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][4] :="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][5] :="zzzzzzzzzzaaaaazzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][6] :="zzzzzzzzzzaaaaazzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][7] :="zzzzzzaazzzzzzzzzzzzzaaazzzzzzzz";
gBitmaps[EXPLOSION][8] :="zzzzzaaaezzzzzzzzzzaaaaaazzzzzzz";
gBitmaps[EXPLOSION][9] :="zzzzaaaezzzzzzzzzzzzzaaaaazzzzzz";
gBitmaps[EXPLOSION][10]:="zzzaaaezzzzzzzzzzzzzzzzaaaazzzzz";
gBitmaps[EXPLOSION][11]:="zeaaaezzzzaaaazzzezzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][12]:="zzeeezzzzaaaazzeezzzzzzzzzaazzzz";
gBitmaps[EXPLOSION][13]:="zzzzzzzzzzzzzpppeeeezzzzzzzaazzz";
gBitmaps[EXPLOSION][14]:="zaaezzzzzzzppeeeeeeeppzzzzzaazzz";
gBitmaps[EXPLOSION][15]:="zaaezzzzzzpeeeeeeeeeeepzzzzaazzz";
gBitmaps[EXPLOSION][16]:="zzaaezzzzzeeeeeeeeeeeepzzzzzzzzz";
gBitmaps[EXPLOSION][17]:="zzaaaezzzpeeeeeeeeeeeepzzzzzzzzz";
gBitmaps[EXPLOSION][18]:="zzzaaezzzpeeeeeeeeeeeepzzzzzzzzz";
gBitmaps[EXPLOSION][19]:="zzzzzzzzzpeeeeeeeeeeeezzzaaazzzz";
gBitmaps[EXPLOSION][20]:="zzzzzzzzzzzeeeeeeeeeepzzzzaaazzz";
gBitmaps[EXPLOSION][21]:="zzzzzzzzzzzzzeeeeeepzzzzzzzaaazz";
gBitmaps[EXPLOSION][22]:="zzzzzzzzzzzzzzzzpppzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][23]:="zzzzzzzaaazzzzzzzeezzzzzzzzzzzzz";
gBitmaps[EXPLOSION][24]:="zzzzzzzzaaazzzzzzeezzzzzzzzzzzzz";
gBitmaps[EXPLOSION][25]:="zzzzzzzzzaaazzzzzzezzzzzzzaaazzz";
gBitmaps[EXPLOSION][26]:="zzzzzzzzzaaazzzzzzezzzzzzaaazzzz";
gBitmaps[EXPLOSION][27]:="zzzzzzzzzzzzzzzzzzezzzzzaaazzzzz";
gBitmaps[EXPLOSION][28]:="zzzzzzzzzzzzzzzzzzzzzzzaaazzzzzz";
gBitmaps[EXPLOSION][29]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][30]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][31]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
gBitmaps[EXPLOSION][32]:="zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

(* 0 *)
gDigitIcons[0][1] := "aaabbaaa";
gDigitIcons[0][2] := "aabbbbaa";
gDigitIcons[0][3] := "abbaabba";
gDigitIcons[0][4] := "abbaabba";
gDigitIcons[0][5] := "abbaabba";
gDigitIcons[0][6] := "abbaabba";
gDigitIcons[0][7] := "aabbbbaa";
gDigitIcons[0][8] := "aaabbaaa";

(* 1 *)
gDigitIcons[1][1] := "aaabbaaa";
gDigitIcons[1][2] := "aabbbaaa";
gDigitIcons[1][3] := "abbbbaaa";
gDigitIcons[1][4] := "aaabbaaa";
gDigitIcons[1][5] := "aaabbaaa";
gDigitIcons[1][6] := "aaabbaaa";
gDigitIcons[1][7] := "aabbbbaa";
gDigitIcons[1][8] := "aabbbbaa";

(* 2 *)
gDigitIcons[2][1] := "aabbbbaa";
gDigitIcons[2][2] := "abbbbbba";
gDigitIcons[2][3] := "abbaabba";
gDigitIcons[2][4] := "aaaabbaa";
gDigitIcons[2][5] := "aaabbaaa";
gDigitIcons[2][6] := "aabbaaaa";
gDigitIcons[2][7] := "abbbbbba";
gDigitIcons[2][8] := "abbbbbba";

(* 3 *)
gDigitIcons[3][1] := "aabbbaaa";
gDigitIcons[3][2] := "abbbbbaa";
gDigitIcons[3][3] := "abaaabba";
gDigitIcons[3][4] := "aaabbbaa";
gDigitIcons[3][5] := "aaabbbaa";
gDigitIcons[3][6] := "abaaabba";
gDigitIcons[3][7] := "abbbbbaa";
gDigitIcons[3][8] := "aabbbaaa";
(* 4 *)
gDigitIcons[4][1] := "aabbaaaa";
gDigitIcons[4][2] := "aabbaaaa";
gDigitIcons[4][3] := "aabbabba";
gDigitIcons[4][4] := "abbaabba";
gDigitIcons[4][5] := "abbbbbba";
gDigitIcons[4][6] := "abbbbbbb";
gDigitIcons[4][7] := "aaaaabba";
gDigitIcons[4][8] := "aaaaabba";
(* 5 *)
gDigitIcons[5][1] := "abbbbbba";
gDigitIcons[5][2] := "abbaaaaa";
gDigitIcons[5][3] := "abbaaaaa";
gDigitIcons[5][4] := "abbbbbaa";
gDigitIcons[5][5] := "aaaabbba";
gDigitIcons[5][6] := "aaaaabba";
gDigitIcons[5][7] := "abbbbbaa";
gDigitIcons[5][8] := "abbbbbaa";
(* 6 *)
gDigitIcons[6][1] := "aaabbaaa";
gDigitIcons[6][2] := "aabbaaaa";
gDigitIcons[6][3] := "abbaaaaa";
gDigitIcons[6][4] := "abbbbbaa";
gDigitIcons[6][5] := "abbaabba";
gDigitIcons[6][6] := "abbaabba";
gDigitIcons[6][7] := "abbaabba";
gDigitIcons[6][8] := "aabbbbaa";
(* 7 *)
gDigitIcons[7][1] := "abbbbbba";
gDigitIcons[7][2] := "abbbbbba";
gDigitIcons[7][3] := "aaaaabba";
gDigitIcons[7][4] := "aaaabbaa";
gDigitIcons[7][5] := "aaaabbaa";
gDigitIcons[7][6] := "aaabbaaa";
gDigitIcons[7][7] := "aaabbaaa";
gDigitIcons[7][8] := "aaabbaaa";
(* 8 *)
gDigitIcons[8][1] := "aabbbbaa";
gDigitIcons[8][2] := "abbaabba";
gDigitIcons[8][3] := "abbaabba";
gDigitIcons[8][4] := "aabbbbaa";
gDigitIcons[8][5] := "aabbbbaa";
gDigitIcons[8][6] := "abbaabba";
gDigitIcons[8][7] := "abbaabba";
gDigitIcons[8][8] := "aabbbbaa";
(* 9 *)
gDigitIcons[9][1] := "aabbbbaa";
gDigitIcons[9][2] := "abbaabba";
gDigitIcons[9][3] := "abbaabba";
gDigitIcons[9][4] := "aabbbbba";
gDigitIcons[9][5] := "aaaaabba";
gDigitIcons[9][6] := "aaaabbaa";
gDigitIcons[9][7] := "aaabbaaa";
gDigitIcons[9][8] := "aabbaaaa";

END ModifiedStandardBitmap.