<* WOFF + *>
IMPLEMENTATION MODULE StandardLib;

FROM IO IMPORT WrStr, WrReal, WrFixReal, WrLn, WrCard, RdKey, RdInt, WrInt;
FROM Lib IMPORT RAND, RANDOM, RANDOMIZE, Delay;
FROM SysClock IMPORT DateTime, CanGetClock, GetClock, Fraction;
FROM Graph IMPORT Init, Line, Disc, Rectangle, Polygon, RawOutText;
FROM Graph IMPORT _clrBLACK, _clrBLUE, _clrGREEN, _clrCYAN, _clrRED, _clrMAGENTA, _clrBROWN, _clrWHITE, _clrGRAY, _clrLIGHTBLUE, _clrLIGHTGREEN, _clrLIGHTCYAN, _clrLIGHTRED, _clrLIGHTMAGENTA, _clrLIGHTYELLOW, _clrBRIGHTWHITE;
FROM RealMath IMPORT arctan, sin, cos, pi, sqrt;

(* CARDINAL procedures *)
(***********************)
  PROCEDURE MaxCard(c1, c2: CARDINAL): CARDINAL;
  BEGIN
    IF c1 >= c2 THEN
      RETURN c1;
    ELSE
      RETURN c2;
    END;
  END MaxCard;

  PROCEDURE MinCard(c1, c2: CARDINAL): CARDINAL;
  BEGIN
    IF c1 <= c2 THEN
      RETURN c1;
    ELSE
      RETURN c2;
    END;
  END MinCard;


(* String manipulation *)
(***********************)
  PROCEDURE StrLength(str: StrAr): CARDINAL;
    VAR i : CARDINAL;
  BEGIN
    i := 0;
    WHILE (str[i] # CHR(0)) AND (i < HIGH(str)) DO
      INC(i);
    END;
    RETURN i;
  END StrLength;

  PROCEDURE StrCopy(VAR strTo : StrAr; strFrom: StrAr);
    VAR i: CARDINAL;
  BEGIN
    FOR i := 0 TO MinCard(HIGH(strTo), HIGH(strFrom)) DO
      strTo[i] := strFrom[i];
    END;
  END StrCopy;

  PROCEDURE StrConcatenate(VAR strTo : StrAr; strFrom: StrAr);
    VAR i, l: CARDINAL;
  BEGIN
    l := StrLength(strTo);
    FOR i := 0 TO MinCard(HIGH(strTo) - l, HIGH(strFrom)) DO
      strTo[l + i] := strFrom[i];
    END;
  END StrConcatenate;

  PROCEDURE InStr(c:CHAR; str:StrAr):BOOLEAN;
    VAR found: BOOLEAN;
      i: CARDINAL;
  BEGIN
    found := FALSE;
    FOR i := 0 TO StrLength(str) - 1 DO
      IF c = str[i] THEN
        found := TRUE;
      END;
    END;
    RETURN found;
  END InStr;

  PROCEDURE StrCompare(str1, str2: StrAr): INTEGER; (* compares 2 strings alphabetically *)
    VAR i: CARDINAL;
  BEGIN
    i := 0;
    WHILE (str1[i] = str2[i]) AND ((i+1) < StrLength(str1)) AND ((i+1) < StrLength(str2)) DO
      INC(i);
    END;
    IF str1[i] < str2[i] THEN
      RETURN -1;
    ELSIF str1[i] > str2[i] THEN
      RETURN 1;
    ELSIF StrLength(str1) < StrLength(str2) THEN
      RETURN -1;
    ELSIF StrLength(str1) > StrLength(str2) THEN
      RETURN 1;
    ELSE
      RETURN 0;
    END;
  END StrCompare;

(* Random generator *)
(********************)
  VAR
    gSeed: CARDINAL;

  CONST PSEUDO_RANDOM_PRIME = 1013;
        INITIAL_SEED = 97;

  PROCEDURE PseudoRandomCardGenerator(): CARDINAL;
    VAR old: CARDINAL;
  BEGIN
    old := gSeed;
    gSeed := (gSeed * 351 + 138) MOD PSEUDO_RANDOM_PRIME;
    IF gSeed = old THEN
      INC(gSeed);
    END;
    RETURN gSeed;
  END PseudoRandomCardGenerator;

  PROCEDURE RandomCard(min, max: CARDINAL): CARDINAL;
  VAR
    factor: REAL;
    c: CARDINAL;
  BEGIN
    IF max < min THEN
      WrStr("ERROR calling RandomCardFromInterval: min ");WrCard(min, 0);WrStr(" is bigger than max ");WrCard(max, 0);WrLn;
      RETURN min;
    ELSE
      IF NOT gPseudoRandom THEN
        RETURN min + RANDOM(max-min+1);
      ELSE
        factor := (FLOAT(max - min) + 0.99999) / FLOAT(PSEUDO_RANDOM_PRIME - 1);
        c := PseudoRandomCardGenerator();
        RETURN min + TRUNC(FLOAT(c) * factor);
      END;
    END;
  END RandomCard;

  PROCEDURE SetSeed(newSeed: CARDINAL);
  BEGIN
    gSeed := newSeed;
  END SetSeed;

(* Clock:  for time measurement and adding delays *)
(*********)

VAR
	start : DateTime;

PROCEDURE StartChrono();
BEGIN
  GetClock(start);
END StartChrono;

PROCEDURE StopChrono(): REAL;
  VAR diff: REAL;
      stop : DateTime;
BEGIN
  GetClock(stop);
  RETURN (FLOAT(stop.hour) - FLOAT(start.hour)) * 3600.
	  + (FLOAT(stop.minute) - FLOAT(start.minute)) * 60.
          + (FLOAT(stop.second) - FLOAT(start.second))
          + (FLOAT(stop.fractions) - FLOAT(start.fractions)) / 100.;	
END StopChrono;

PROCEDURE Wait(time : REAL);   (* in seconds, precision: about 1 ms *)
(* I had some problems with the delay, when it was 0. So use this procedure instead *)
BEGIN
  IF time > 0.0 THEN
    Delay(VAL(CARDINAL, time * 1000.));
  END;
END Wait;

(* Graph *)
(*********)
 PROCEDURE Square(x0,y0,size,colour:CARDINAL);
 BEGIN
   Rectangle (x0 - size/2,y0 - size/2,x0 + size/2,y0 + size/2, colour,TRUE);
 END Square;

 PROCEDURE Asterisk(x0,y0,size,colour:CARDINAL);
   VAR a:CARDINAL;
 BEGIN
   a:= VAL(CARDINAL,((VAL(REAL,size)/(2.0))*0.7071) ); (* cos(45)*)
   Line(x0-size/2, y0, x0+size/2,y0, colour);
   Line(x0, y0+size/2, x0, y0-size/2, colour);
   Line(x0-a, y0+a, x0+a, y0-a, colour);
   Line(x0+a, y0+a, x0-a, y0-a, colour);
 END Asterisk;

 PROCEDURE Triangle(x0,y0,size,colour:CARDINAL);
   VAR a:CARDINAL;
 BEGIN
   a:= VAL(CARDINAL,((VAL(REAL,size)/(2.0))*0.7071) ); (* cos(45)*)
   Line(x0, y0-a, x0-size/2, y0+a, colour);
   Line(x0-size/2, y0+a, x0+size/2, y0+a, colour);
   Line(x0, y0-a, x0+size/2, y0+a, colour);
 END Triangle;

PROCEDURE Arrow(x0, y0, x1, y1, size, colour: CARDINAL);
  CONST ANGLE = 0.3;
  VAR alfa: REAL;
       xa, ya: ARRAY[1..3] OF CARDINAL;
BEGIN
  Line(x0, y0, x1, y1, colour);
  alfa := AngleWithXAx(x0, y0, x1, y1);
  xa[1] := x1;
  ya[1] := y1;
  xa[2] := VAL(INTEGER, x1) - VAL(INTEGER, FLOAT(size) * cos(alfa + ANGLE));
  ya[2] := VAL(INTEGER, y1) - VAL(INTEGER, FLOAT(size) * sin(alfa + ANGLE));
  xa[3] := VAL(INTEGER, x1) - VAL(INTEGER, FLOAT(size) * cos(alfa - ANGLE));
  ya[3] := VAL(INTEGER, y1) - VAL(INTEGER, FLOAT(size) * sin(alfa - ANGLE));
  Polygon(3, xa, ya, colour, TRUE);
END Arrow;

PROCEDURE Distance(x1, y1, x2, y2: INTEGER): CARDINAL;
BEGIN
 RETURN VAL(CARDINAL, sqrt((VAL(REAL, x1) - VAL(REAL, x2))*(VAL(REAL, x1) - VAL(REAL, x2)) + (VAL(REAL, y1) - VAL(REAL, y2))*(VAL(REAL, y1) - VAL(REAL, y2))));
END Distance;

PROCEDURE RadiansToDegrees(angle:REAL): INTEGER;
BEGIN
  RETURN VAL(INTEGER, angle * 180.0/pi);
END RadiansToDegrees;

PROCEDURE DegreesToRadians(angle:INTEGER):REAL;
BEGIN
  RETURN VAL(REAL, angle) * pi/180.0;
END DegreesToRadians;

PROCEDURE RotatePoint(VAR x, y:INTEGER; x0, y0: INTEGER; angle: REAL);
   (* rotate point (x, y) around (x0, y0) with clockwise angle in radians *)
   VAR x_:INTEGER;
BEGIN
  x_ := VAL(INTEGER, VAL(REAL, x0) + (VAL(REAL, x) - VAL(REAL, x0)) * cos(angle) - (VAL(REAL, y)-VAL(REAL, y0)) * sin(angle));
  y := VAL(INTEGER, VAL(REAL, y0) +  (VAL(REAL, x) - VAL(REAL, x0)) * sin(angle) + (VAL(REAL, y)-VAL(REAL, y0)) * cos(angle));
  x := x_;
END RotatePoint;

PROCEDURE RotatePointDegrees(VAR x, y:INTEGER; x0, y0: INTEGER; angle: INTEGER);
BEGIN
  RotatePoint(x, y, x0, y0, DegreesToRadians(angle));
END RotatePointDegrees;

PROCEDURE AngleWithXAx(x0, y0, x1, y1: INTEGER):REAL;
  VAR alfa: REAL;
BEGIN
 IF x1 # x0 THEN
     alfa := arctan((FLOAT(y1)-FLOAT(y0))/(FLOAT(x1)-FLOAT(x0)));
     IF (x0 > x1) THEN
       alfa := alfa + pi;
     END;
   ELSE
     IF y1 > y0 THEN
       alfa := pi/2.0;
     ELSE
       alfa := -pi/2.0;
     END;
   END;
RETURN alfa;
END AngleWithXAx;

PROCEDURE AngleWithXAx_REAL(x0, y0, x1, y1: REAL):REAL;
  VAR alfa: REAL;
BEGIN
 IF x1 # x0 THEN
     alfa := arctan((y1-y0)/(x1-x0));
     IF (x0 > x1) THEN
       alfa := alfa + pi;
     END;
   ELSE
     IF y1 > y0 THEN
       alfa := pi/2.0;
     ELSE
       alfa := -pi/2.0;
     END;
   END;
RETURN alfa;
END AngleWithXAx_REAL;

PROCEDURE TestColours();
(* to test if colours on your screen are right *)
  VAR x: CHAR;
BEGIN
	
	RawOutText(20, 20, _clrBLACK, "This is colour _clrBLACK");
  RawOutText(20, 40, _clrBLUE, "This is colour _clrBLUE");
  RawOutText(20, 60, _clrGREEN, "This is colour _clrGREEN");
  RawOutText(20, 80, _clrCYAN, "This is colour _clrCYAN");
  RawOutText(20, 100, _clrRED, "This is colour _clrRED");
  RawOutText(20, 120, _clrMAGENTA, "This is colour _clrMAGENTA");
  RawOutText(20, 140, _clrBROWN, "This is colour _clrBROWN");
  RawOutText(20, 160, _clrWHITE, "This is colour _clrWHITE");
  RawOutText(20, 180, _clrGRAY, "This is colour _clrGRAY");
  RawOutText(20, 200, _clrLIGHTBLUE, "This is colour _clrLIGHTBLUE");
  RawOutText(20, 220, _clrLIGHTGREEN, "This is colour _clrLIGHTGREEN");
  RawOutText(20, 240, _clrLIGHTCYAN, "This is colour _clrLIGHTCYAN");
  RawOutText(20, 260, _clrLIGHTRED, "This is colour _clrLIGHTRED");
  RawOutText(20, 280, _clrLIGHTMAGENTA, "This is colour _clrLIGHTMAGENTA");
  RawOutText(20, 300, _clrLIGHTYELLOW, "This is colour _clrLIGHTYELLOW");
  RawOutText(20, 320, _clrBRIGHTWHITE, "This is colour _clrBRIGHTWHITE");
  RawOutText(20, 400, _clrBRIGHTWHITE, "Push any key to finish the program");
	WrStr("Push any key to finish the program");
  x := RdKey(); WrLn;
END TestColours;



(* Various *)
(***********)
PROCEDURE RdArrowKey(): CARDINAL;
  (* Met "een pijl" krijg je 2 chars, de eerste is 0, de tweede is:
     UP 72 'H'  RIGHT 77 'M' DOWN 80 'P' LEFT 75 'K' *)
   VAR
     x: CHAR;
     dir: CARDINAL;
 BEGIN
     x := RdKey();
     IF (ORD(x) # 0) THEN
       dir := 4; (* not an arrow key *)
     ELSE
       x := RdKey();
       CASE ORD(x) OF
         72: dir := 0;
         | 77: dir := 1;
         | 80: dir := 2;
         | 75: dir := 3;
         ELSE dir := 4; (* not an arrow key *)
       END;
     END;
     RETURN dir;
 END RdArrowKey;

(********** INITIALISATION **********)
BEGIN
  RANDOMIZE; (* Initialise the Modula-2 random generator *)
  gPseudoRandom := TRUE;
  gSeed := INITIAL_SEED; (* Initialise our pseudo random generator *)

END StandardLib.
