<* WOFF + *>
IMPLEMENTATION MODULE StandardSound;
(* Needed for sound support *)
IMPORT SYSTEM;
IMPORT Windows;
IMPORT FIO;

FROM IO IMPORT RdKey, WrStr, WrCard, WrLn, WrInt, WrChar, WrReal;
FROM Lib IMPORT Delay;

(* STEP 2: specify here the folder of your sound files *)
CONST
	SOUND_FILES_FOLDER = "sounds\\"; (* FOLDER OF SOUND FILES !!!!!!!!!! *)
TYPE
	SoundRc = RECORD
			fileName : ARRAY [1..100] OF CHAR;
			soundFlag : CARDINAL;
		  END;	
		
VAR
	gActive : BOOLEAN; (* sound is working *)
	gSoundList : ARRAY SoundEventEn OF SoundRc;

  (* necessary procedures *)
  PROCEDURE MinCard(c1, c2: CARDINAL): CARDINAL;
  BEGIN
    IF c1 <= c2 THEN
      RETURN c1;
    ELSE
      RETURN c2;
    END;
  END MinCard;

  PROCEDURE StrLength(str: ARRAY OF CHAR): CARDINAL;
    VAR i : CARDINAL;
  BEGIN
    i := 0;
    WHILE (str[i] # CHR(0)) AND (i < HIGH(str)) DO
      INC(i);
    END;
    RETURN i;
  END StrLength;

  PROCEDURE StrCopy(VAR strTo : ARRAY OF CHAR; strFrom: ARRAY OF CHAR);
    VAR i: CARDINAL;
  BEGIN
    FOR i := 0 TO MinCard(HIGH(strTo), HIGH(strFrom)) DO
      strTo[i] := strFrom[i];
    END;
  END StrCopy;

  PROCEDURE StrConcatenate(VAR strTo : ARRAY OF CHAR; strFrom: ARRAY OF CHAR);
    VAR i, l: CARDINAL;
  BEGIN
    l := StrLength(strTo);
    FOR i := 0 TO MinCard(HIGH(strTo) - l, HIGH(strFrom)) DO
      strTo[l + i] := strFrom[i];
    END;
  END StrConcatenate;

  (* Ticker tape display emulation *)
  PROCEDURE TickerTape (txt : ARRAY OF CHAR; length : CARDINAL; time : REAL);
  VAR
	i, j : CARDINAL;
  BEGIN

  WrStr(txt);

  FOR i := 0 TO length DO
	Delay(VAL(CARDINAL, time * 1000.0));
	WrChar(CHR(13));
	FOR j := 1 TO i DO
		WrChar(' ');
	END;
	FOR j := 0 TO HIGH(txt) DO
		WrChar(txt[j]);
	END;
  END;
	
  END TickerTape;

	
(* Play the sound associated with the given SoundEventEn *)
PROCEDURE PlaySoundEvent(e: SoundEventEn);
	VAR string : ARRAY [1..100] OF CHAR;
BEGIN
IF NOT gActive THEN
	RETURN; (* No sounds -> no use *)
END;

StrCopy(string, SOUND_FILES_FOLDER);
StrConcatenate(string, gSoundList[e].fileName);
(* Play the sound *)
IF (NOT Windows.PlaySound (SYSTEM.ADR(string), NIL, gSoundList[e].soundFlag)) THEN
   (* it seems it was not possible to play the sound *)
	IF NOT FIO.Exists(string) THEN
	  WrStr("** Sound file "); WrStr(gSoundList[e].fileName);WrStr("  need here! "); WrLn;
	  WrStr("** Check whether you th file in the sounds directory ");WrStr(SOUND_FILES_FOLDER);	WrLn;
	ELSE
	  WrStr("** Unknown Sound error with file "); WrStr(gSoundList[e].fileName);WrStr(", sound is desactived."); WrLn;
	  gActive := FALSE;
	END;
END;	

END PlaySoundEvent;


(* Initialize the different sound settings *)
PROCEDURE InitSound(asynchronousSound: BOOLEAN) : BOOLEAN;	
CONST
	OFFSET = 10;
	INTERVAL = 0.02;
	STD_FLAGS = Windows.SND_FILENAME;
	XTR_FLAGS = Windows.SND_FILENAME + Windows.SND_NOSTOP;
VAR
	string : ARRAY [1..100] OF CHAR;
	i : CARDINAL;
	e : SoundEventEn;
	soundFlag : CARDINAL;
BEGIN

(* Is asynchronous sound requested *)
IF (asynchronousSound) THEN
	soundFlag := Windows.SND_ASYNC;
ELSE
	soundFlag := Windows.SND_SYNC;
END;
(*WrLn;
WrStr("You should here sound now, this to check whether sound works on your system..."); WrLn;
WrLn; *)

StrCopy(string, SOUND_FILES_FOLDER);
StrConcatenate(string, "intro.wav");
(* Play the test sound here *)
gActive := Windows.PlaySound (SYSTEM.ADR(string), NIL, Windows.SND_FILENAME + Windows.SND_ASYNC);

(* Check whether sound works *)
IF (NOT gActive) THEN
	WrStr("Sound can NOT be used on your system!"); WrLn;
	WrStr("All sound events will be desactivated."); WrLn;
	WrLn;

	RETURN gActive;
ELSE
(*	WrLn; WrLn; WrLn;
	TickerTape("                ", OFFSET, INTERVAL); WrLn;
	TickerTape("                ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                        ", OFFSET, INTERVAL); WrLn;
	TickerTape("                        ", OFFSET, INTERVAL); WrLn;
	TickerTape("                      ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	TickerTape("                          ", OFFSET, INTERVAL); WrLn;
	WrLn;
	TickerTape("", OFFSET, INTERVAL); WrLn;
	TickerTape("", OFFSET, INTERVAL); WrLn;
	WrLn; WrLn;

	WrStr("Sound works fine!"); WrLn; *)
END;

(* STEP 3: associate sound events with wav-files *)

(* Initialize the sound arrays *)
gSoundList[ATTACK1].fileName := "char1.wav";
gSoundList[ATTACK1].soundFlag := STD_FLAGS + soundFlag;

gSoundList[ATTACK2].fileName := "char2.wav";
gSoundList[ATTACK2].soundFlag := STD_FLAGS + soundFlag;

gSoundList[EXPLOSION1].fileName := "exp1.wav";
gSoundList[EXPLOSION1].soundFlag := STD_FLAGS + soundFlag;

gSoundList[EXPLOSION2].fileName := "exp2.wav";
gSoundList[EXPLOSION2].soundFlag := STD_FLAGS + soundFlag;

gSoundList[DEATH1].fileName := "die1.wav";
gSoundList[DEATH1].soundFlag := STD_FLAGS + soundFlag;

gSoundList[DEATH2].fileName := "die2.wav";
gSoundList[DEATH2].soundFlag := STD_FLAGS + soundFlag;

gSoundList[MOVE1].fileName := "move1.wav";
gSoundList[MOVE1].soundFlag := STD_FLAGS + soundFlag;

gSoundList[MOVE2].fileName := "move2.wav";
gSoundList[MOVE2].soundFlag := STD_FLAGS + soundFlag;

gSoundList[INTRO].fileName := "intro.wav";
gSoundList[INTRO].soundFlag := STD_FLAGS + soundFlag;

gSoundList[STILTE].fileName := "stilte.wav";
gSoundList[STILTE].soundFlag := STD_FLAGS + soundFlag;

gSoundList[TRUMPET].fileName := "trumpet2.wav";
gSoundList[TRUMPET].soundFlag := STD_FLAGS + soundFlag;

RETURN gActive;
END InitSound;

END StandardSound.
